import os

from django.contrib.auth import get_user_model
from django.core.urlresolvers import reverse
from django.test import TestCase


class ViewsTest(TestCase):
    fixtures = [
        'initial_data.json',
        'uploader_test_data.json',
    ]

    def setUp(self):
        credentials = {
            'username': 'staff_user',
            'password': 'testing!',
        }

        staff_user = get_user_model()._default_manager.create_user(
            username=credentials['username'],
            password=credentials['password'],
            email='superuser@example.com',
            first_name='Super',
            last_name='User',
        )

        staff_user.is_staff = True
        staff_user.is_superuser = True
        staff_user.save()

        self.client.login(
            username=credentials['username'],
            password=credentials['password'],
        )

        self.tests_path = os.path.join(
            os.path.dirname(os.path.abspath(__file__)),
            'uploads',
        )

    def test_fileupload_views(self):
        tests = [
            {
                'url': 'admin:uploader_fileupload_changelist',
                'url_args': [],
            },
            {
                'url': 'admin:uploader_fileupload_add',
                'url_args': [],
            },
            {
                'url': 'admin:uploader_fileupload_change',
                'url_args': [1],
            },
            {
                'url': 'admin:uploader_fileupload_delete',
                'url_args': [1],
            },
        ]

        for test in tests:
            url = reverse(test['url'], args=test['url_args'])
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200)

    def test_fileupload_add(self):
        url = reverse('admin:uploader_fileupload_add')

        with open(
            os.path.join(self.tests_path, 'some_text_file.txt'),
            'rb'
        ) as upload:
            response = self.client.post(url, {
                'name': 'Some Text File',
                'file': upload,
                '_save': 'Save',
            })

        self.assertEqual(response.status_code, 302)

    def test_imageupload_views(self):
        tests = [
            {
                'url': 'admin:uploader_imageupload_changelist',
                'url_args': [],
            },
            {
                'url': 'admin:uploader_imageupload_add',
                'url_args': [],
            },
            {
                'url': 'admin:uploader_imageupload_change',
                'url_args': [1],
            },
            {
                'url': 'admin:uploader_imageupload_delete',
                'url_args': [1],
            },
        ]

        for test in tests:
            url = reverse(test['url'], args=test['url_args'])
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200)

    def test_imageupload_add(self):
        url = reverse('admin:uploader_imageupload_add')

        with open(
            os.path.join(
                self.tests_path,
                'pixabay.com-landscape-nature-view-forest-pond-215588.jpg'
            ),
            'rb'
        ) as upload:
            response = self.client.post(url, {
                'name': 'Some Image',
                'image': upload,
                'resize_image': ['1'],
                '_save': 'Save',
            })

        self.assertEqual(response.status_code, 302)

    def test_thumbnailsize_views(self):
        tests = [
            {
                'url': 'admin:uploader_thumbnailsize_changelist',
                'url_args': [],
            },
            {
                'url': 'admin:uploader_thumbnailsize_add',
                'url_args': [],
            },
            {
                'url': 'admin:uploader_thumbnailsize_change',
                'url_args': [1],
            },
            {
                'url': 'admin:uploader_thumbnailsize_delete',
                'url_args': [1],
            },
        ]

        for test in tests:
            url = reverse(test['url'], args=test['url_args'])
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200)



from django.test import TestCase

from ..models import ImageUpload, ThumbnailSize


class SignalsTest(TestCase):
    fixtures = [
        'initial_data.json',
        'uploader_test_data.json',
    ]

    def test_image_upload_post_init(self):
        image_upload = ImageUpload.objects.get(pk=1)
        image_upload.resize_image.add(ThumbnailSize.objects.get(pk=1))
        self.assertEqual(image_upload.thumbnail_set.count(), 0)
        image_upload = ImageUpload.objects.get(pk=1)

        # Verify that initializing the image upload caused the
        # `force_thumbnails` receiver function to execute.

        self.assertEqual(image_upload.thumbnail_set.count(), 1)

    def test_image_upload_post_save(self):
        image_upload = ImageUpload.objects.get(pk=1)
        image_upload.resize_image.add(ThumbnailSize.objects.get(pk=1))
        self.assertEqual(image_upload.thumbnail_set.count(), 0)
        image_upload.save()

        # Verify that saving the image upload caused the
        # `create_thumbnails` receiver function to execute.

        self.assertEqual(image_upload.thumbnail_set.count(), 1)



from django.db import models

UL_PATH = 'uploads/%Y/%m/%d/'


class ThumbnailSize(models.Model):
    name = models.CharField(max_length=30)
    height = models.PositiveIntegerField()
    width = models.PositiveIntegerField()

    def as_size(self):
        return u"%sx%s" % (self.width, self.height)

    def __unicode__(self):
        return u'%s (%ix%i)' % (self.name, self.width, self.height)

    class Meta:
        unique_together = (('height', 'width'),)
        ordering = ('width', 'height')


class Thumbnail(models.Model):
    upload = models.ForeignKey('ImageUpload')
    size = models.ForeignKey(ThumbnailSize)
    image = models.ImageField(upload_to='%simages/' % UL_PATH, max_length=255)

    def __unicode__(self):
        try:
            return u'%s (%ix%i)' % (self.upload,
                                    self.image.width,
                                    self.image.height)
        except:
            return u'%s (%s)' % (self.upload, self.size.name)

    class Meta:
        unique_together = (('upload', 'size'),)


class ImageUpload(models.Model):
    name = models.CharField(max_length=50)
    image = models.ImageField(upload_to='%simages/' % UL_PATH, max_length=255)
    resize_image = models.ManyToManyField(ThumbnailSize, blank=True, help_text='Choose the size or sizes you would like the system to create for you. Choose none of the sizes to preserve the dimensions of the image you are uploading.')

    """
    def __init__(self, *args, **kwargs):
        super(ImageUpload, self).__init__(*args, **kwargs)

        if self.id and not self.original_name:
            self.original_name = self.image.name
            self.save()
    """

    def __unicode__(self):
        return self.name

    def get_absolute_url(self):
        return self.image.url


class FileUpload(models.Model):
    name = models.CharField(max_length=50)
    file = models.FileField(upload_to='%sfiles/' % UL_PATH, max_length=255)

    def __unicode__(self):
        return self.name

    def get_absolute_url(self):
        return self.file.url



from django.conf import settings

default_settings = {
    'enable_force_thumbnails_receiver': True,
}

UPLOADER_SETTINGS = {}


def compute_settings():
    for name, value in default_settings.items():
        UPLOADER_SETTINGS[name] = value

    if hasattr(settings, 'UPLOADER'):
        UPLOADER_SETTINGS.update(settings.UPLOADER)

compute_settings()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='FileUpload',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=50)),
                ('file', models.FileField(max_length=255, upload_to=b'uploads/%Y/%m/%d/files/')),
            ],
        ),
        migrations.CreateModel(
            name='ImageUpload',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=50)),
                ('image', models.ImageField(max_length=255, upload_to=b'uploads/%Y/%m/%d/images/')),
            ],
        ),
        migrations.CreateModel(
            name='Thumbnail',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('image', models.ImageField(max_length=255, upload_to=b'uploads/%Y/%m/%d/images/')),
            ],
        ),
        migrations.CreateModel(
            name='ThumbnailSize',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=30)),
                ('height', models.PositiveIntegerField()),
                ('width', models.PositiveIntegerField()),
            ],
            options={
                'ordering': ('width', 'height'),
            },
        ),
        migrations.AlterUniqueTogether(
            name='thumbnailsize',
            unique_together=set([('height', 'width')]),
        ),
        migrations.AddField(
            model_name='thumbnail',
            name='size',
            field=models.ForeignKey(to='uploader.ThumbnailSize'),
        ),
        migrations.AddField(
            model_name='thumbnail',
            name='upload',
            field=models.ForeignKey(to='uploader.ImageUpload'),
        ),
        migrations.AddField(
            model_name='imageupload',
            name='resize_image',
            field=models.ManyToManyField(help_text=b'Choose the size or sizes you would like the system to create for you. Choose none of the sizes to preserve the dimensions of the image you are uploading.', to='uploader.ThumbnailSize', blank=True),
        ),
        migrations.AlterUniqueTogether(
            name='thumbnail',
            unique_together=set([('upload', 'size')]),
        ),
    ]



from django.conf import settings
from django.conf.urls import include, url
from django.conf.urls.static import static
from django.contrib import admin
from django.views.generic import TemplateView

urlpatterns = [
    url(r'^$', TemplateView.as_view(template_name='homepage.html')),
    url(r'^accounts/', include('django.contrib.auth.urls')),
    url(r'^admin/', include(admin.site.urls)),
    url(r'^uploader/', include('uploader.urls')),
] + static(
    settings.MEDIA_URL, document_root=settings.MEDIA_ROOT
) + static(
    settings.STATIC_URL, document_root=settings.STATIC_ROOT
)



from collections import OrderedDict

from django.contrib.auth import get_user_model
from django.core.urlresolvers import reverse
from django.test import TestCase


class ViewsTest(TestCase):
    fixtures = [
        'uploader_test_data.json',
    ]

    def get_user_types(self):
        for user_type, credentials in self.user_types.items():
            if credentials:
                self.client.login(**credentials)

            yield user_type

    def setUp(self):
        normal_credentials = {
            'username': 'normal_user',
            'password': 'testing!',
        }

        get_user_model()._default_manager.create_user(
            username=normal_credentials['username'],
            password=normal_credentials['password'],
            email='normal@example.com',
            first_name='Normal',
            last_name='User',
        )

        staff_credentials = {
            'username': 'staff_user',
            'password': 'testing!',
        }

        staff_user = get_user_model()._default_manager.create_user(
            username=staff_credentials['username'],
            password=staff_credentials['password'],
            email='staff@example.com',
            first_name='Staff',
            last_name='User',
        )

        staff_user.is_staff = True
        staff_user.save()

        self.user_types = OrderedDict([
            ('anonymous', None),
            ('normal', normal_credentials),
            ('staff', staff_credentials),
        ])

    def test_link_list(self):
        url = '/uploader/link_list.js'

        for user_type in self.get_user_types():
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200)

    def test_link_list_old_style(self):
        url = '/uploader/link_list-old_style.js'

        for user_type in self.get_user_types():
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200)

    def test_image_list(self):
        url = '/uploader/image_list.js'

        for user_type in self.get_user_types():
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200)

    def test_image_list_old_style(self):
        url = '/uploader/image_list-old_style.js'

        for user_type in self.get_user_types():
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200)

    def test_all(self):
        url = reverse('uploader-all')

        for user_type in self.get_user_types():
            if user_type == 'staff':
                expected_status_code = 200
            else:
                expected_status_code = 302

            response = self.client.get(url)
            self.assertEqual(response.status_code, expected_status_code)

    def test_thumbnail(self):
        url = reverse('uploader-create-thumbnail', args=[
            1,
            240,
            180,
        ])

        for user_type in self.get_user_types():
            if user_type == 'staff':
                expected_status_code = 200
            else:
                expected_status_code = 302

            response = self.client.get(url)
            self.assertEqual(response.status_code, expected_status_code)



#!/usr/bin/env python
from setuptools import setup, find_packages
from uploader import __version__

setup(
    name='danemco-uploader',
    version=__version__,
    description="A file and image uploader",
    author="Danemco, LLC",
    author_email='dev@velocitywebworks.com',
    url='https://git.velocitywebworks.com/lib/danemco-uploader',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'Django>=1.8',
        'Pillow',
    ],
)



from django.conf.urls import url

from .views import LinkListView, ImageListView, all_files, create_thumbnail

urlpatterns = [
    url(r'^link_list\.js$', LinkListView.as_view()),
    url(r'^link_list-old_style\.js$', LinkListView.as_view(use_old_output_style=True)),
    url(r'^image_list\.js$', ImageListView.as_view()),
    url(r'^image_list-old_style\.js$', ImageListView.as_view(use_old_output_style=True)),
]

# for the admin
urlpatterns += [
    url(r'^all/$', all_files, name="uploader-all"),
    url(r'^thumbnail/(\d+)/(\d+)x(\d+)/$', create_thumbnail, name="uploader-create-thumbnail")
]



from django.db.models.signals import post_init, post_save

from .models import ImageUpload, Thumbnail
from .settings import UPLOADER_SETTINGS
from .utils import make_thumbnail


def create_thumbnails(sender, instance, created, **kwargs):
    """
    Creates the thumbnails for an image upload
    """
    for size in instance.resize_image.all():
        tn = make_thumbnail(
            instance.image.url,
            width=size.width,
            height=size.height,
        )

        if tn:
            thumb, created = Thumbnail.objects.get_or_create(
                size=size,
                upload=instance,
            )

            thumb.image = tn
            thumb.save()


def force_thumbnails(sender, instance, **kwargs):
    """
    Forces an image upload to be resaved when it doesn't have any thumbnails
    """
    if instance.id and instance.thumbnail_set.count() == 0:
        instance.save()


post_save.connect(create_thumbnails, sender=ImageUpload)

if UPLOADER_SETTINGS['enable_force_thumbnails_receiver']:
    post_init.connect(force_thumbnails, sender=ImageUpload)
